---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Asis; use Asis;

with Ada.Characters.Handling;
with Ada.Exceptions;

with Asis.Exceptions;
with Asis.Errors;
with Asis.Implementation;

with Asis.Elements;

with Ast2Cfg.Transformation;
with Ast2Cfg.Output; use Ast2Cfg.Output;

package body Ast2Cfg.Actuals_For_Traversing is

   -------------
   -- Post_Op --
   -------------

   procedure Post_Op
     (Element :        Asis.Element;
      Control : in out Asis.Traverse_Control;
      State   : in out Traversal_State) is
   begin

      Transformation.Handle_Post(Element, Control, State);

   exception

      when Ex : Asis.Exceptions.ASIS_Inappropriate_Context  |
        Asis.Exceptions.ASIS_Inappropriate_Container        |
        Asis.Exceptions.ASIS_Inappropriate_Compilation_Unit |
        Asis.Exceptions.ASIS_Inappropriate_Element          |
        Asis.Exceptions.ASIS_Inappropriate_Line             |
        Asis.Exceptions.ASIS_Inappropriate_Line_Number      |
        Asis.Exceptions.ASIS_Failed                         =>

         Put(Error, "[Post_Op] ERROR: ASIS exception (");
         Put(Error, Ada.Exceptions.Exception_Name(Ex));
         Put(Error, ") is raised");
         New_Line(Error);

         Put(Error, "ASIS Error Status is ");
         Put(Error, Asis.Errors.Error_Kinds'Image(Asis.Implementation.Status));
         New_Line(Error);

         Put(Error, "ASIS Diagnosis is ");
         New_Line(Error);
         Put(Error, Ada.Characters.Handling.To_String(Asis.Implementation.Diagnosis));
         New_Line(Error);

         Asis.Implementation.Set_Status;
         raise;

      when Ex : others =>

         Put(Error, "[Post_Op] ERROR: ");
         Put(Error, Ada.Exceptions.Exception_Name(Ex));
         Put(Error, " is raised (");
         Put(Error, Ada.Exceptions.Exception_Information(Ex));
         Put(Error, ")");
         New_Line(Error);
         raise;

   end Post_Op;

   ------------
   -- Pre_Op --
   ------------

   procedure Pre_Op
     (Element :        Asis.Element;
      Control : in out Asis.Traverse_Control;
      State   : in out Traversal_State) is
   begin

      Transformation.Handle_Pre(Element, Control, State);

   exception

      when Ex : Asis.Exceptions.ASIS_Inappropriate_Context  |
        Asis.Exceptions.ASIS_Inappropriate_Container        |
        Asis.Exceptions.ASIS_Inappropriate_Compilation_Unit |
        Asis.Exceptions.ASIS_Inappropriate_Element          |
        Asis.Exceptions.ASIS_Inappropriate_Line             |
        Asis.Exceptions.ASIS_Inappropriate_Line_Number      |
        Asis.Exceptions.ASIS_Failed                         =>

         Put(Error, "[Pre_Op] ERROR: ASIS exception (");
         Put(Error, Ada.Exceptions.Exception_Name(Ex));
         Put(Error, ") is raised");
         New_Line(Error);

         Put(Error, "ASIS Error Status is ");
         Put(Error, Asis.Errors.Error_Kinds'Image(Asis.Implementation.Status));
         New_Line(Error);

         Put(Error, "ASIS Diagnosis is ");
         New_Line(Error);
         Put(Error, Ada.Characters.Handling.To_String(Asis.Implementation.Diagnosis));
         New_Line(Error);

         Asis.Implementation.Set_Status;
         raise;

      when Ex : others =>

         Put(Error, "[Pre_Op] ERROR: ");
         Put(Error, Ada.Exceptions.Exception_Name(Ex));
         Put(Error, " is raised (");
         Put(Error, Ada.Exceptions.Exception_Information(Ex));
         Put(Error, ")");
         New_Line(Error);
         raise;

   end Pre_Op;

end Ast2Cfg.Actuals_For_Traversing;
