---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Exceptions;
with Ada.Containers;

package Ast2Cfg.Declaration_Id is

   type Object is private;
   Null_Declaration_Id: constant Object;
   
   procedure Init (O: in out Object);
   
   function "=" (Left: in Object; Right: in Object) return Boolean; -- test for equality of 2 ids,
                                                                      -- /= is implicitly available

   function "<" (Left: in Object; Right: in Object) return Boolean;

   function ">" (Left: in Object; Right: in Object) return Boolean;

   function To_String (O: in Object) return String; -- get a string representation of the id
   function Hash (O: in Object) return Ada.Containers.Hash_Type; -- return the hash of an id
   
   Declaration_Id_Overflow: exception; -- too many declarations
      
private

   Last_Id: Long_Long_Natural := 0;
      
   type Object is
     record
	Id: Long_Long_Natural := 0;
     end record;
   
   Null_Declaration_Id: constant Object := (Id => 0);
   
end Ast2Cfg.Declaration_Id;
