---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Containers;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Hashed_Sets;
with Ada.Unchecked_Deallocation;

with Ast2Cfg.CFGs;
with Ast2Cfg.Nodes;
with Ast2Cfg.Id;
with Ast2Cfg.Union_Find;
with Ast2Cfg.Post_Trans_Processing; use Ast2Cfg.Post_Trans_Processing;

package Ast2Cfg.Loop_Refinement is

   procedure Refine (CFG: in out CFGs.CFG_Object'Class);

private

   -- fill Dfs_Sorted, Id_Dfs and Backedges
   procedure Dfs (Cur: in Nodes.Node_Class_Ptr; Last: in Long_Long_Natural);

   -- Tarjan's algorithm for finding loops; Loop refinement
   procedure Refine_Loops;

   -- hash function for use with the Vertex_Map container
   function Hash (X: in Long_Long_Natural) return Ada.Containers.Hash_Type;

   package Vertex_Map is new Ada.Containers.Hashed_Maps (Key_Type => Long_Long_Natural,
                                                         Element_Type => Long_Long_Natural,
                                                         Hash => Hash,
                                                         Equivalent_Keys => "=");
   use type Vertex_Map.Cursor;

   package Vertex_Set is new Ada.Containers.Hashed_Sets (Element_Type => Ast2Cfg.Long_Long_Natural,
                                                         Hash => Hash,
                                                         Equivalent_Elements => "=");
   use type Vertex_Set.Cursor;

   package Id_To_Dfs_Num is new Ada.Containers.Hashed_Maps (Key_Type => Id.Pointer,
                                                            Element_Type => Long_Long_Natural,
                                                            Hash => Id.Hash,
                                                            Equivalent_Keys => Id."=");

   type Node_Array is array (Long_Long_Natural range <>) of Nodes.Node_Class_Ptr;
   type Node_Array_Ptr is access Node_Array;
   procedure Free is new Ada.Unchecked_Deallocation (Node_Array, Node_Array_Ptr);

   Node_Count    : Long_Long_Natural := 0; -- the current dfs number
   Dfs_Sorted    : Node_Array_Ptr := null; -- the nodes sorted in dfs order
   Id_Dfs        : Id_To_Dfs_Num.Map; -- maps the Id of a node back to the dfs number
   Backedges     : Vertex_Map.Map; -- the backedges (m,n) where n is the key of the map
   Dangling_Roots: Post_Trans_Processing.Node_Set.Set;


end Ast2Cfg.Loop_Refinement;
