---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008,2009 Raul Fechete, Georg Kienesberger    --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Ast2Cfg.Id;
with Ast2Cfg.Generic_List;
with Ast2Cfg.Generic_Pointer_List;

package Ast2Cfg.Utils is

   -- Types
   package UStr_List      is new Generic_List (Item => Unbounded_String);
   -- a little hack, to avoid more nasty ones
   package UStr_List_List is new Generic_Pointer_List (Item_Pointer => UStr_List.Object,
                                                       Free_Procedure => UStr_List.Clear,
                                                       "=" => Ustr_List."="); -- to avoid use type clause

   -- Convert a Wide String to Unbounded String an Trim it
   function W2u(Str: in Wide_String) return Unbounded_String;

   -- Convert a String to Unbounded String
   function S2u(Str: in String) return Unbounded_String;

   -- Convert a Unbounded_String to String
   function U2s(U_Str: in Unbounded_String) return String;

   -- Convert a Wide String to String
   function W2s(Str: in Wide_String) return String;

   -- Mask "s and \s in a String with \
   -- Warning: The String may increase in length!
   function Mask(Str: in String) return String;

   -- Mask "s and \s in a Unbounded_String with \
   -- Warning: The String may increase in length!
   function Mask(U_Str: in Unbounded_String) return Unbounded_String;

   -- Tokenize a String
   function Tokenize(Str: in String; Token: in String)
                    return UStr_List.Object;

   function Merge_UStr_List(UString_List: in UStr_List.Object; Separator: in String) return String;
   function Merge_UStr_List(UString_List: in UStr_List.Object; Separator: in String) return Unbounded_String;
   function Merge_UStr_List_List (UString_List_List: in UStr_List_List.Object;
                                  Separator1: in String; Separator2: in String) return String;

   -- Return an Image of an UStr_List
   function UStr_List_Dump (List: in UStr_List.Object) return String;

end Ast2Cfg.Utils;
